@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 1992/3 Theo de Raadt <deraadt@fsa.ca>
- * Modifications 1994 Olaf Kirch <okir@monad.swb.de>
+ * Modifications (c) 1994 Olaf Kirch <okir@monad.swb.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -28,20 +28,27 @@
  * SUCH DAMAGE.
  */
 
-#include <stdio.h>
-#include <getopt.h>
-#include <string.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <limits.h>
 #include <netdb.h>
 #include <time.h>
-#include <sys/types.h>
-#include <pwd.h>
-#include <errno.h>
 #include <unistd.h>
-#include <limits.h>
+#include <string.h>
+#include <stdio.h>
+#ifdef NO_GETOPT_H
+#include <getopt.h>
+#endif
+#include <errno.h>
+#include <pwd.h>
+#ifdef __sgi
+# include <crypt.h>
+#endif
+
 #include <rpc/rpc.h>
 #include <rpcsvc/yp_prot.h>
 #include <rpcsvc/ypclnt.h>
-#include <rpcsvc/yppasswd.h>
+#include "yppasswd.h"
 
 char	*prog_name;
 uid_t	uid;
@@ -81,10 +88,19 @@ getnewpasswd(register struct passwd *pw)
         return NULL;
     }
 #ifndef DEBUG
-    if (strlen(p) <= 5 && (uid != 0 || ++tries < 2)) {
+    if (strlen(p) <= 5 && (uid != 0 || ++tries < 6)) {
         printf("Please enter a longer password.\n");
         continue;
     }
+#ifdef USE_OBVIOUS
+    if (isobvious(p) && (uid != 0 || ++tries < 6)) {
+	printf(
+	"This password might be too easy for a program to guess. Please\n"
+	"use a more complicated one, e.g. mixing upper- and lower-case\n"
+	"letters, digits and special characters.\n");
+	continue;
+    }
+#endif
 #endif
     strcpy(buf, p);
     p = getpass("Please retype new password:");
@@ -299,17 +315,17 @@ main(int argc, char **argv)
   uid = getuid();
   if (user == NULL) {
       if ((pw = getpwuid(uid)) == NULL) {
-          fprintf ( stderr, "%s: unknown user (uid=%d).\n", prog_name, uid );
+          fprintf(stderr, "%s: unknown user (uid=%d).\n", prog_name, (int) uid);
           exit(1);
       }
   } else {
       if ((pw = getpwnam(user)) == NULL) {
-          fprintf ( stderr, "%s: unknown user: %s.\n", prog_name, user );
+          fprintf(stderr, "%s: unknown user: %s.\n", prog_name, user);
           exit(1);
       }
       if (pw->pw_uid != uid && uid != 0) {
-          fprintf ( stderr, "%s: Only root may change account information "
-          		    "for others\n", prog_name );
+          fprintf(stderr, "%s: Only root may change account information "
+          		    "for others\n", prog_name);
           exit(1);
       }
   }
@@ -324,7 +340,7 @@ main(int argc, char **argv)
   yppasswd.newpw.pw_shell = pw->pw_shell;
   yppasswd.oldpass = NULL;
   
-  switch (opt_passwd + (opt_fullname << 1) + (opt_shell << 2)) {
+  switch(opt_passwd + (opt_fullname << 1) + (opt_shell << 2)) {
   case 1:
       what = "YP password";
       break;
@@ -345,10 +361,15 @@ main(int argc, char **argv)
 
     sprintf (prompt, "Please enter %spassword:", opt_passwd? "old " : "");
     s = getpass (prompt);
+
+    /* We can't check the password with shadow passwords enabled. We
+     * leave the cheking to yppasswdd */
+#ifndef SHADOWPWD
     if( strcmp(crypt(s, pw->pw_passwd), pw->pw_passwd)) {
         fprintf(stderr, "Sorry.\n");
         exit (1);
     }
+#endif
     yppasswd.oldpass = strdup(s);
   }
 
@@ -378,8 +399,8 @@ main(int argc, char **argv)
   bzero( (char*)&status, sizeof(status) );
   timeout.tv_sec = 25; timeout.tv_usec = 0;
   err = clnt_call( clnt, YPPASSWDPROC_UPDATE,
-  		 xdr_yppasswd, (char*)&yppasswd,
-  		 xdr_int,      (char*)&status,
+  		 (xdrproc_t) xdr_yppasswd, (char*) &yppasswd,
+  		 (xdrproc_t) xdr_int,      (char*) &status,
   		 timeout );
 
   if (err) {
